@@ -44,7 +44,7 @@ __PACKAGE__->add_columns
 __PACKAGE__->set_primary_key('Version');
 
 package DBICVersion::Schema;
-use base 'DBICTest::BaseSchema';
+use base 'DBIx::Class::Schema';
 use strict;
 use warnings;
 
@@ -52,7 +52,7 @@ our $VERSION = '3.0';
 
 __PACKAGE__->register_class('Table', 'DBICVersion::Table');
 __PACKAGE__->load_components('+DBIx::Class::Schema::Versioned');
-__PACKAGE__->upgrade_directory("t/var/versioning_ddl-$$");
-__PACKAGE__->backup_directory("t/var/versioning_backup-$$");
+__PACKAGE__->upgrade_directory('t/var/');
+__PACKAGE__->backup_directory('t/var/backup/');
 
 1;
